import { Body, Controller, Post, Res, UploadedFile, UseGuards, UseInterceptors } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { FileInterceptor } from '@nestjs/platform-express';
import { ApiTags } from '@nestjs/swagger';
import { memoryStorage } from 'multer';
import { PreventCache } from '../shared/preventcache.decorator';
import { FileService } from '../shared/services/file.service';
import { FileRotateDto } from './dto/file-rotate.dto';
import { MoveFileDto } from './dto/move-file.dto';
import { ReadFileDto } from './dto/read-file.dto';
import { RemoveDto } from './dto/remove.dto';
import { WriteFileToDirectoryDto } from './dto/write-file-to-directory.dto';

@Controller('files')
@ApiTags('Fichiers')
export class FilesController {
  constructor(
    private fileService: FileService, //private configService: ConfigService
  ) {}

  // *********************************************************************************************
  // * READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ *
  // * READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ READ *
  // *********************************************************************************************

  @Post('read-file')
  @UseGuards(AuthGuard('api-key'))
  @PreventCache()
  async readFile(@Body() dto: ReadFileDto, @Res() res): Promise<void> {
    res.send(Buffer.from(await this.fileService.readFile(dto.filepath)));
  }

  // *********************************************************************************************
  // * WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE *
  // * WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE WRITE *
  // *********************************************************************************************

  @Post('write-file-to-directory')
  @UseGuards(AuthGuard('api-key'))
  @PreventCache()
  @UseInterceptors(FileInterceptor('file', { storage: memoryStorage() }))
  async writeFileToDirectory(@UploadedFile() file: Express.Multer.File, @Body() dto: WriteFileToDirectoryDto, @Res() res): Promise<void> {
    await this.fileService.writeFileToDirectory(dto.filepath, dto.fullName, file.buffer);
    res.send();
  }

  @Post('move-doc')
  @UseGuards(AuthGuard('api-key'))
  @PreventCache()
  async moveDoc(@Body() dto: MoveFileDto, @Res() res): Promise<void> {
    await this.fileService.moveDoc(dto.srcPath, dto.destPath);
    res.send();
  }

  // *********************************************************************************************
  // * REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOV *
  // * REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOVE REMOV *
  // *********************************************************************************************

  @Post('remove')
  @UseGuards(AuthGuard('api-key'))
  @PreventCache()
  remove(@Body() dto: RemoveDto, @Res() res): void {
    this.fileService.remove(dto.path);
    res.send();
  }

  // *********************************************************************************************
  // * VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS V *
  // * VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS VARIOUS V *
  // *********************************************************************************************

  @Post('file-rotate')
  @UseGuards(AuthGuard('api-key'))
  @PreventCache()
  async fileRotate(@Body() dto: FileRotateDto, @Res() res): Promise<void> {
    await this.fileService.fileRotate(dto.folder, dto.durationDays);
    res.send();
  }
}
