import { GroupKey, Utils } from './utils';

describe('Utils', () => {
  describe('A) Nominal case', () => {
    it('1) Should execute isNumber', () => {
      // verify
      expect(Utils.isNumber(4)).toBeTruthy();
      expect(Utils.isNumber(4.5)).toBeTruthy();
      expect(Utils.isNumber(-4)).toBeTruthy();
      expect(Utils.isNumber(false)).toBeFalsy();
      expect(Utils.isNumber('string')).toBeFalsy();
      expect(Utils.isNumber('4')).toBeFalsy();
    });
    it('2) Should execute isString', () => {
      // verify
      expect(Utils.isString(4)).toBeFalsy();
      expect(Utils.isString(4.5)).toBeFalsy();
      expect(Utils.isString(-4)).toBeFalsy();
      expect(Utils.isString(false)).toBeFalsy();
      expect(Utils.isString('string')).toBeTruthy();
      expect(Utils.isString('4')).toBeTruthy();
    });

    it('4) Should execute normalizeString with undefined string', () => {
      // prepare
      let str: string;
      // verify
      expect(Utils.normalizeString(str)).toBeUndefined();
    });

    it('5) Should execute normalizeString', () => {
      // prepare
      const str: string = 'Madame Loïc a déménagé à la Bretagne.';
      // verify
      const expectStr: string = 'Madame Loic a demenage a la Bretagne.';
      expect(Utils.normalizeString(str)).toBe(expectStr);
    });

    it('6) Should execute normalizeFilename with undefined string', () => {
      // prepare
      let str: string;
      // verify
      expect(Utils.normalizeFilename(str)).toBeUndefined();
    });

    it('7) Should execute normalizeFilename', () => {
      // prepare
      const str: string = 'Madame Loïc a déménagé à la Bretagne.';
      // verify
      const expectStr: string = 'Madame_Loïc_a_déménagé_à_la_Bretagne.';
      expect(Utils.normalizeFilename(str)).toBe(expectStr);
    });

    it('7) Should execute range', () => {
      // prepare
      const years: Number[] = [2017, 2018, 2019, 2020, 2021];
      // execute
      const result: Number[] = Utils.range(2017, 2021);
      // verify
      expect(result).toEqual(years);
    });

    type Person = { firstname: string; lastname: string; age: number };
    type Position = { position: string };

    it('8) Should pick', () => {
      // prepare
      const value: Person & Position = {
        firstname: 'John',
        lastname: 'Doe',
        age: 35,
        position: 'employee',
      };
      // execute
      const result: any = Utils.pick(value, ['lastname', 'firstname']);
      // verify
      expect(result).toEqual({ lastname: 'Doe', firstname: 'John' });
    });

    it('9) Should group by', () => {
      // prepare
      const items: (Person & Position)[] = [
        {
          firstname: 'John',
          lastname: 'Doe',
          age: 35,
          position: 'employee',
        },
        {
          firstname: 'Jane',
          lastname: 'Doe',
          age: 26,
          position: 'executive cadre',
        },
        {
          firstname: 'Thomas',
          lastname: 'Anderson',
          age: 24,
          position: 'employee',
        },
      ];
      // execute
      const results: GroupKey<Person & Position, 'position'>[] = Utils.groupByKey(items, 'position');
      // verify
      expect(results).toHaveLength(2);
      expect(results[0].key).toEqual({ position: 'employee' });
      expect(results[0].items).toHaveLength(2);
      expect(results[0].items).toEqual([
        {
          firstname: 'John',
          lastname: 'Doe',
          age: 35,
          position: 'employee',
        },
        {
          firstname: 'Thomas',
          lastname: 'Anderson',
          age: 24,
          position: 'employee',
        },
      ]);

      expect(results[1].key).toEqual({ position: 'executive cadre' });
      expect(results[1].items).toHaveLength(1);
      expect(results[1].items).toEqual([
        {
          firstname: 'Jane',
          lastname: 'Doe',
          age: 26,
          position: 'executive cadre',
        },
      ]);
    });

    it('10) Normalize search', () => {
      // execute
      const result: string[] = Utils.normalizeSearch('test de mots');
      // verify
      expect(result).toEqual(['%test%', '%mots%']);
    });

    it('11a) should get time (hours + minutes) from minutes', () => {
      //prepare
      //execute
      const result: string = Utils.getTimeFromMins(1426);

      //verify
      expect(result).toBe('23h46');
    });

    it('11b) should get time (hours + minutes) from undefined value', () => {
      //prepare
      //execute
      const result: string = Utils.getTimeFromMins(undefined);

      //verify
      expect(result).toBe('0h00');
    });

    it('12) should format phone', () => {
      expect(Utils.formatPhone(undefined)).toBe(undefined);
      expect(Utils.formatPhone(null)).toBe(null);
      expect(Utils.formatPhone('')).toBe('');
      expect(Utils.formatPhone('a123456789')).toBe('a123456789');
      expect(Utils.formatPhone('0123456789')).toBe('01 23 45 67 89');
      expect(Utils.formatPhone('0123456789', '.')).toBe('01.23.45.67.89');
    });
  });
});
